/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2003 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package org.netbeans.modules.editor;

import java.awt.FlowLayout;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.text.JTextComponent;
import org.netbeans.editor.BaseKit;
import org.netbeans.editor.SideBarFactory;
import org.netbeans.editor.Utilities;
import org.netbeans.modules.editor.options.BaseOptions;
import org.openide.cookies.InstanceCookie;
import org.openide.loaders.DataObject;

/**
 *  Editor Customizable Side Bar.
 *  Contains components for particular MIME type as defined in XML layer.
 *
 *  @author  Martin Roskanin
 */
public class CustomizableSideBar extends JPanel {
    
    private static HashMap components = new HashMap(5);
    
    public static final String FOLDER_NAME = "SideBar"; //NOI18N
    
    public CustomizableSideBar(JTextComponent target){
        BoxLayout bl = new javax.swing.BoxLayout(this, javax.swing.BoxLayout.X_AXIS);
        setLayout(bl);
        
        JComponent panelComponents[] = getPanelComponents(target);
        for (int i=0; i<panelComponents.length; i++){
            add(panelComponents[i]);
        }
    }
    
    private List getInstanceCookiesPerKitClass(Class kitClass){
        ArrayList retList = new ArrayList();   
        if (kitClass==null) return retList;
        synchronized (components){
            if (components.containsKey(kitClass)){
                return (List)components.get(kitClass);
            }else{

                BaseKit kit = BaseKit.getKit(kitClass);
                String name = kit.getContentType();
                if (name == null) {
                    return retList; //empty
                }

                BaseOptions bo = BaseOptions.getOptions(kitClass);
                if (bo==null) return retList; //empty

                List files = bo.getOrderedMultiPropertyFolderFiles(FOLDER_NAME);

                for (int i=0; i<files.size(); i++){
                    if (!(files.get(i) instanceof DataObject)) continue;

                    DataObject dob = (DataObject) files.get(i);
                    InstanceCookie ic = (InstanceCookie)dob.getCookie(InstanceCookie.class);
                    if (ic!=null){
                        try{
                            if (SideBarFactory.class.isAssignableFrom(ic.instanceClass() )){
                                retList.add(ic);
                            }
                        }catch(IOException ioe){
                            ioe.printStackTrace();
                        }catch(ClassNotFoundException cnfe){
                            cnfe.printStackTrace();
                        }
                    }
                }
                components.put(kitClass, retList);
                return retList;
            }
        }
    }
    
    private JComponent[] getPanelComponents(JTextComponent target){
        List retList = new ArrayList();
        List icList = getInstanceCookiesPerKitClass(Utilities.getKitClass(target));

        try{
            for (int i = 0; i<icList.size(); i++){
                InstanceCookie ic = (InstanceCookie)icList.get(i);
                Object obj = ic.instanceCreate();
                retList.add(((SideBarFactory)obj).createSideBar(target));
            }
        }catch(IOException ioe){
            ioe.printStackTrace();
        }catch(ClassNotFoundException cnfe){
            cnfe.printStackTrace();
        }
        
        JComponent ret[] = new JComponent[retList.size()];
        retList.toArray(ret);
        return ret;
    }
    
}
